import json
import cv2
import numpy
from matplotlib import pyplot as plt

# returns the x and y coordinate of one openPose point
# videoName: name of the video without path i.e. 'example.mp4'
# frameNumber: the number of the frame you want to find the point in starting from 0
# model: the body part, possible options: 'pose', 'right_hand', 'left_hand', 'face'
# pointNumber: the number of the point
# openPoseFolder: the folder where the openPose output is
def getPosition(videoName, frameNumber, model, pointNumber, openPoseFolder):
    #get full path of openPose file
    jsonFile=openPoseFolder+'/'+videoName.split('.')[0]+'_'+str(int(frameNumber)).zfill(12)+'_keypoints.json'
    #open the openPose file and load the data
    json_data=open(jsonFile)
    data = json.load(json_data)
    #get data of first person (we are assuming there is only one in the video)
    p = data["people"]

    if len(p) > 0:
        #get points of the choosen body part
        lm = p[0][model+"_keypoints_2d"]
        #find the choosen point and return it. For each point there are 3 numbers: x and y coordinate and a confidence value.
        return((lm[pointNumber*3], lm[pointNumber*3+1]))
    #print error message if there is no person found in video
    else:
        print('Error: no person found in video')

# returns the confidence value for one openPose point
# videoName: name of the video without path i.e. 'example.mp4'
# frameNumber: the number of the frame you want to find the point in starting from 0
# model: the body part, possible options: 'pose', 'right_hand', 'left_hand', 'face'
# pointNumber: the number of the point
# openPoseFolder: the folder where the openPose output is
def getConfidence(videoName, frameNumber, model, pointNumber, openPoseFolder):
    # get full path of openPose file
    jsonFile = openPoseFolder + '/' + videoName.split('.')[0] + '_' + str(int(frameNumber)).zfill(
        12) + '_keypoints.json'
    # open the openPose file and load the data
    json_data = open(jsonFile)
    data = json.load(json_data)
    # get data of first person (we are assuming there is only one in the video)
    p = data["people"]

    if len(p) > 0:
        # get points of the choosen body part
        lm = p[0][model + "_keypoints_2d"]
        # find the choosen point and return it. For each point there are 3 numbers: x and y coordinate and a confidence value.
        return (lm[pointNumber * 3+2])
    # print error message if there is no person found in video
    else:
        print('Error: no person found in video')



#displays one point in one frame of the video
# videoName: name of the video without path i.e. 'example.mp4'
# frameNumber: the number of the frame you want to find the point in starting from 0
# point: x and y coordinate of point you want to display
# videoFolder: folder where the video is stored
def showPointInImage(videoName, frameNumber, point, videoFolder):
    # open the video
    cap = cv2.VideoCapture(videoFolder+'/'+videoName)
    # read first frame
    ret, frame = cap.read()

    #forward to correct frame by reading frame by frame
    for i in range(frameNumber):
        ret, frame = cap.read()

    #round point to whole number so that it refers to one pixel
    point=(round(point[0]), round(point[1]))
    #draw the point
    cv2.circle(frame, point, 5, (0, 0, 255), -1)

    # Display the resulting frame
    fig =plt.gcf()
    fig.set_dpi(120)
    plt.imshow(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
    plt.show()

    # When everything done, release the capture
    cap.release()

# returns how many frames there are in the video
# videoName: name of the video without path i.e. 'example.mp4'
# videoFolder: folder where the video is stored
def getNumberOfFrames(videoName, videoFolder):
    # open video
    cap = cv2.VideoCapture(videoFolder+'/'+videoName)
    # return number of frames
    return int(cap.get(cv2.CAP_PROP_FRAME_COUNT))

#displays all points in one frame of the video
# videoName: name of the video without path i.e. 'example.mp4'
# frameNumber: the number of the frame you want to find the point in starting from 0
# openPoseFolder: the folder where the openPose output is
# videoFolder: folder where the video is stored
def showAllPointsInImage(videoName, frameNumber, openPoseFolder, videoFolder):

    cap = cv2.VideoCapture(videoFolder+'/'+videoName)
    ret, frame = cap.read()
    # Read the next frame from the video.
    for i in range(frameNumber):
        ret, frame = cap.read()

    size=4
    color=(0,0,255)
    bodypart='pose'
    for p in range(25):
        point = getPosition(videoName, frameNumber, bodypart, p, openPoseFolder)
        #round point to whole number so that it refers to one pixel
        point=(round(point[0]), round(point[1]))

        cv2.circle(frame, point, size, color, -1)
    bodypart='face'
    for p in range(70):
        point = getPosition(videoName, frameNumber, bodypart, p, openPoseFolder)
        #round point to whole number so that it refers to one pixel
        point=(round(point[0]), round(point[1]))

        cv2.circle(frame, point, size, color, -1)
    bodypart='hand_left'
    for p in range(21):
        point = getPosition(videoName, frameNumber, bodypart, p, openPoseFolder)
        #round point to whole number so that it refers to one pixel
        point=(round(point[0]), round(point[1]))

        cv2.circle(frame, point, size, color, -1)
    bodypart='hand_right'
    for p in range(21):
        point = getPosition(videoName, frameNumber, bodypart, p, openPoseFolder)
        #round point to whole number so that it refers to one pixel
        point=(round(point[0]), round(point[1]))

        cv2.circle(frame, point, size, color, -1)
   # cv2.imwrite('allPoints.png', frame)
    # Display the resulting frame
    fig =plt.gcf()
    fig.set_dpi(120)
    plt.imshow(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
    plt.show()
    # When everything done, release the capture
    cap.release()


# displays how one point moves within one video
# videoName: name of the video without path i.e. 'example.mp4'
# bodyPart: the body part, possible options: 'pose', 'right_hand', 'left_hand', 'face'
# pointNumber: the number of the point
# openPoseFolder: the folder where the openPose output is
# videoFolder: folder where the video is stored
def followPoint(videoName, bodyPart,pointNumber,  openPoseFolder, videoFolder):

	#open video to get number of frames and frame size
    cap = cv2.VideoCapture(videoFolder+'/'+videoName)
    ret, frame = cap.read()
    numFrames=int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
    
	#get positions of all points    
    xpos=[]
    ypos=[]
    c=numpy.arange(numFrames)
    for i in range(numFrames):
        point=getPosition(videoName, i, bodyPart, pointNumber, openPoseFolder)
        xpos.append(point[0])
        ypos.append(point[1])
        
    #draw points as scatter plot
    plt.scatter(xpos,ypos, c=c, cmap='viridis')
    # set plot properties
    plt.colorbar()
    plt.xlabel('pixel position in x direction')
    plt.ylabel('pixel position in y direction')
    axes = plt.gca()
    axes.set_xlim([0, frame.shape[1]])
    axes.set_ylim([0, frame.shape[0]])
    plt.gca().invert_yaxis()
    fig =plt.gcf()
    fig.set_dpi(100)
    #fig.set_size_inches(6.8, 4)
    plt.show()
    
# get position and confidence values for all points within one body part and one video
# videoName: name of the video without path i.e. 'example.mp4'
# bodyPart: the body part, possible options: 'pose', 'right_hand', 'left_hand', 'face'
# openPoseFolder: the folder where the openPose output is
# videoFolder: folder where the video is stored
# return value is a matrix with dimensions frames x points x 3 (x-coordinate, y-coordinate, confidence)
def getAllOpenPoseData(videoName, bodyPart, openPoseFolder, videoFolder):
    numberOfFrames=getNumberOfFrames(videoName, videoFolder)
    allData=[]
    for i in range(numberOfFrames):
        #get full path of openPose file
        jsonFile=openPoseFolder+'/'+videoName.split('.')[0]+'_'+str(int(i)).zfill(12)+'_keypoints.json'
        #open the openPose file and load the data
        json_data=open(jsonFile)
        data = json.load(json_data)
        #get data of first person (we are assuming there is only one in the video)
        p = data["people"]

        if len(p) > 0:
            #get points of the choosen body part
            lm = p[0][bodyPart+"_keypoints_2d"]
            #append it to allData seperately for each point
            allData.append([])
            for j in range(int(len(lm)/3)):
                allData[i].append([lm[j*3], lm[j*3+1], lm[j*3+2]])
        #print error message if there is no person found in video
        else:
            print('Error: no person found in video')
    return numpy.array(allData)